/**
 * @jest-environment jsdom
 */
import React from "react";
import { render } from "@testing-library/react";
import Dropdown from "../components/elements/Dropdown";
import DropdownItem from "../components/elements/DropdownItem";
/*
Helper function to make maintaince easier
*/
function renderDropdownButton(props = {}) {
    const defaultProps = {
        children: React.createElement(DropdownItem, { title: 'Dropdown Item', onClick: (x) => { x; } }),
        closeDropdown: (x = 1) => { x; }
    };
    return render(React.createElement(Dropdown, Object.assign({}, defaultProps, props)));
}
describe("<Dropdown />", () => {
    test("Should render a non-null Dropdown", async () => {
        const { getByText } = renderDropdownButton();
        const dropdownItem = await getByText("Dropdown Item");
        expect(dropdownItem).not.toBeNull();
    });
    test("Should call the callback on a click", async () => {
        const dropdownItemOnClick = jest.fn((x) => { x; });
        const { getByText } = renderDropdownButton({ children: React.createElement(DropdownItem, { title: 'Dropdown Item', onClick: dropdownItemOnClick }) });
        const dropdownItem = await getByText("Dropdown Item");
        dropdownItem.click();
        // Check it was called
        expect(dropdownItemOnClick.mock.calls.length).toBe(1);
    });
    // TODO: Test that the Dropdown component's closeDropdown function
    // gets called automatically on any click.
});
//# sourceMappingURL=Dropdown.test.js.map