/**
 * @jest-environment jsdom
 */
import React from "react";
import { render } from "@testing-library/react";
import Mito, { DataTypeInMito } from "../components/Mito";
import MitoAPI from "../api";
import { StepType } from "../types/StepTypes";
import { DFSource } from "../types/DFSources";
jest.mock('../api'); // MitoAPI is not a mock constructor!
// Mock the CSS to get rid of it, as for some reason it does not load properly
jest.mock('ag-grid-community/dist/styles/ag-grid.css', () => jest.fn());
jest.mock('ag-grid-community/dist/styles/ag-theme-alpine.css', () => jest.fn());
// To make sure they are all consistent, we define the data
// that goes in the Mito sheet, and then derive the rest of the 
// variables we need for Mito from it!
const sheetJSONArray = [
    {
        columnIDs: ['a', 'b', 'c'],
        index: ['1', '2'],
        // TODO: check that this is the right shape
        data: [['1', '2'], ['3, 4'], ['5', '6']]
    }
];
const sheetShapeArray = sheetJSONArray.map(sheetJson => {
    return {
        rows: sheetJson.index.length,
        cols: sheetJson.columnIDs.length
    };
});
const dfNames = sheetJSONArray.map((sheetJson, index) => {
    return `df${index}`;
});
const dfSources = sheetJSONArray.map(() => {
    return DFSource.Passed;
});
const columnIDsMapArray = sheetJSONArray.map((sheetJson) => {
    const columnIds = {};
    sheetJson.columnIDs.forEach(columnHeader => {
        columnIds[columnHeader] = '';
    });
    return columnIds;
});
const columnSpreadsheetCodeJSONArray = sheetJSONArray.map((sheetJson) => {
    const columnSpreadsheetCodeJSON = {};
    sheetJson.columnIDs.forEach(columnHeader => {
        columnSpreadsheetCodeJSON[columnHeader] = '';
    });
    return columnSpreadsheetCodeJSON;
});
const columnTypeJSONArray = sheetJSONArray.map((sheetJson) => {
    const columnTypeJSON = {};
    sheetJson.columnIDs.forEach(columnHeader => {
        columnTypeJSON[columnHeader] = "number_series" /* NUMBER_SERIES */;
    });
    return columnTypeJSON;
});
const columnFiltersArray = sheetJSONArray.map((sheetJson) => {
    const columnTypeJSON = {};
    sheetJson.columnIDs.forEach(columnID => {
        columnTypeJSON[columnID] = {
            operator: 'And',
            filters: []
        };
    });
    return columnTypeJSON;
});
const mitoAPI = new MitoAPI(jest.fn(x => x), jest.fn(), jest.fn());
/*
  Helper function to make maintaince easier
*/
function renderMito(props = {}) {
    const defaultProps = {
        mitoAPI: mitoAPI,
        dfNames: dfNames,
        columnIDsMapArray: columnIDsMapArray,
        dfSources: dfSources,
        sheetShapeArray: sheetShapeArray,
        columnSpreadsheetCodeJSONArray: columnSpreadsheetCodeJSONArray,
        sheetJSONArray: sheetJSONArray,
        savedAnalysisNames: [],
        columnFiltersArray: columnFiltersArray,
        columnTypeJSONArray: columnTypeJSONArray,
        model_id: '123',
        isLocalDeployment: true,
        userEmail: 'n@g.com',
        shouldUpgradeMitosheet: false,
        stepSummaryList: [{
                step_id: 'initialize',
                step_idx: 0,
                step_type: StepType.Initialize,
                step_display_name: 'Created a Mitosheet',
                step_description: 'Created a Mitosheet',
            }],
        currStepIdx: 0,
        tourJSON: {
            typeOfDataInTool: DataTypeInMito.PERSONAL,
            receivedTours: []
        },
        numFeedbacks: 0,
        numUsages: 0,
    };
    return render(React.createElement(Mito, Object.assign({}, defaultProps, props)));
}
describe("<Mito />", () => {
    test("Should render a Mito sheet", async () => {
        const { baseElement } = renderMito();
        const mitoContainer = baseElement.children[0].children[0];
        expect(mitoContainer).not.toBeNull();
    });
});
//# sourceMappingURL=Mito.test.js.map