/**
 * @jest-environment jsdom
 */
import React from "react";
import { render } from "@testing-library/react";
import Select from "../components/elements/Select";
/*
Helper function to make maintaince easier
*/
function renderSelect(props = {}) {
    const defaultProps = {
        value: 'optionOne',
        options: ['optionOne', 'optionTwo', 'optionThree'],
        onChange: (option) => { return option; }
    };
    return render(React.createElement(Select, Object.assign({}, defaultProps, props)));
}
describe("<Select />", () => {
    test("Should render a non-null Select", async () => {
        const { getByText } = renderSelect();
        const select = await getByText("optionOne");
        expect(select).not.toBeNull();
    });
    test("Should call the callback on a click", async () => {
        const selectItemCallback = jest.fn((x) => { x; });
        const { getByText } = renderSelect({ onChange: selectItemCallback });
        // Open the Select's Dropdown
        const select = await getByText("optionOne");
        select.click();
        // Select option 2
        const optionToSelect = await getByText("optionTwo");
        optionToSelect.click();
        // Check it was called
        expect(selectItemCallback.mock.calls.length).toBe(1);
        expect(selectItemCallback.mock.calls[0][0]).toBe("optionTwo");
    });
});
//# sourceMappingURL=Select.test.js.map