// Copyright (c) Mito
import React, { Fragment } from 'react';
/*
    Import CSS that we use globally, list these in alphabetical order
    to make it easier to confirm we have imported all sitewide css.

    Except we put the colors.css first because it creates variables used elsewhere.
*/
import '../../css/sitewide/colors.css';
import '../../css/sitewide/ag-grid-overwrites.css';
import '../../css/sitewide/all-modals.css';
import '../../css/sitewide/animations.css';
import '../../css/sitewide/buttons.css';
import '../../css/sitewide/element-sizes.css';
import '../../css/sitewide/flexbox.css';
import '../../css/sitewide/fonts.css';
import '../../css/sitewide/height.css';
import '../../css/sitewide/margins.css';
import '../../css/sitewide/paddings.css';
import '../../css/sitewide/text.css';
import '../../css/sitewide/widths.css';
import '../../css/sitewide/icons.css';
import { getNewSelectedColumn, MITO_INDEX } from '../utils/gridData';
// Import helpers
import { getColumnWidthsArray } from '../utils/gridStyling';
// Import sheet and code components
import MitoSheet from './MitoSheet';
import Footer from './footer/Footer';
import FormulaBar from './FormulaBar';
import Toolbar from './toolbar/Toolbar';
import LoadingIndicator from './LoadingIndicator';
// Import modals
import ErrorModal from './modals/ErrorModal';
import SaveAnalysisOverwriteModal from './modals/SaveAnalysisOverwriteModal';
import ReplayImportModal from './modals/ReplayImportModal';
import PivotTaskpane from './taskpanes/PivotTable/PivotTaskpane';
import { EDITING_TASKPANES, TaskpaneType, WIDE_TASKPANES } from './taskpanes/taskpanes';
import MergeTaskpane from './taskpanes/Merge/MergeTaskpane';
import DocumentationTaskpane from './taskpanes/Documentation/DocumentationTaskpane';
import ControlPanelTaskpane, { ControlPanelTab } from './taskpanes/ControlPanel/ControlPanelTaskpane';
import SignUpModal from './modals/SignupModal';
import UpgradeModal from './modals/UpgradeModal';
import StepsTaskpane from './taskpanes/Steps/StepsTaskpane';
import CatchUpPopup from './CatchUpPopup';
import ReplayAnalysisTaskpane from './taskpanes/ReplayAnalysis/ReplayAnalysisTaskpane';
import ImportTaskpane from './taskpanes/Import/ImportTaskpane';
import Tour from './tour/Tour';
import FeedbackTaskpane, { FeedbackOpenLocation, getCurrentFeedbackQuestionAndPlaceholder } from './taskpanes/Feedback/FeedbackTaskpane';
import FeedbackThanksTaskpane from './taskpanes/Feedback/FeedbackThanksTaskpane';
import { TourName } from './tour/Tours';
import GraphSidebar from './taskpanes/Graph/GraphSidebar';
import { DFSource } from '../types/DFSources';
import DownloadTaskpane from './taskpanes/Download/DownloadTaskpane';
import SaveTaskpane from './taskpanes/Save/SaveTaskpane';
import ClearAnalysisModal from './modals/ClearAnalysisModal';
import { ModalEnum } from './modals/modals';
import { CellEditingMode } from './MitoCellEditor';
// NOTE: this should be the same as the file in the Python code
// which is in data_in_mito.py
export var DataTypeInMito;
(function (DataTypeInMito) {
    DataTypeInMito["PERSONAL"] = "personal";
    DataTypeInMito["TUTORIAL"] = "tutorial";
    DataTypeInMito["PROVIDED"] = "provided";
    DataTypeInMito["NONE"] = "none";
})(DataTypeInMito || (DataTypeInMito = {}));
class Mito extends React.Component {
    constructor(props) {
        super(props);
        let formulaBarValue = '';
        let selectedColumnID = '';
        // Set the default values for the formula bar and selected column (if possible, aka the sheet has some data in it)
        if (this.props.sheetJSONArray.length > 0) {
            formulaBarValue = this.props.sheetJSONArray[0].data[0] === undefined
                ? '' : this.props.sheetJSONArray[0].data[0][0];
            selectedColumnID = getNewSelectedColumn(this.props.sheetJSONArray[0].columnIDs);
        }
        // Build the default column widths
        const defaultColumnWidthsArray = getColumnWidthsArray(this.props.sheetJSONArray).columnWidthsArray;
        /*
            If the user has not input their email, then we default the modal info to the
            email collection modal. Otherwise, if the user should update, we default to
            the upgrade modal.

            If neither of these conditons are true, then we don't default to any modal at
            all.
        */
        const defaultModalInfo = this.props.userEmail == '' ? { type: ModalEnum.SignUp } : (this.props.shouldUpgradeMitosheet ? { type: ModalEnum.Upgrade } : { type: ModalEnum.None });
        /*
            First, we check if there is feedback that we want the user to give
            and we then we popup the feedback taskpane automatically in this case.

            Otherwise, if we don't pass any data to the sheet, we automatically
            open the import modal.
        */
        const currOpenTaskpane = getCurrentFeedbackQuestionAndPlaceholder(props.numFeedbacks, props.numUsages) !== undefined
            ? { type: TaskpaneType.FEEDBACK, openLocation: FeedbackOpenLocation.AUTO_POPUP }
            : (props.sheetJSONArray.length > 0 ? { type: TaskpaneType.NONE } : { type: TaskpaneType.IMPORT });
        this.state = {
            dfNames: this.props.dfNames,
            dfSources: this.props.dfSources,
            columnIDsMapArray: this.props.columnIDsMapArray,
            sheetShapeArray: this.props.sheetShapeArray,
            columnSpreadsheetCodeJSONArray: this.props.columnSpreadsheetCodeJSONArray,
            sheetJSONArray: this.props.sheetJSONArray,
            columnFiltersArray: this.props.columnFiltersArray,
            columnTypeJSONArray: this.props.columnTypeJSONArray,
            selectedSheetIndex: 0,
            formulaBarValue: formulaBarValue,
            selectedColumnID: selectedColumnID,
            selectedRowIndex: 0,
            /*
                we tell if we're in cell editing mode through the editingMode state variable

                the editingColumnID and editingRowIndex are used to update the formula of the correct
                cell editor. They get set to the new values when we turn cell editing mode back on, but are unchanged
                when we turn cell editing mode off so that we can handle formulas that error by letting the user
                edit the formula that they submitted.
            */
            editingModeState: {
                editingMode: CellEditingMode.OFF,
                editingSheetIndex: 0,
                editingColumnID: "",
                editingRowIndex: -1,
                lastFormula: formulaBarValue,
                formulaCursorIndex: 2,
            },
            movingColumnParams: undefined,
            documentationOpen: false,
            errorJSON: {
                event: '',
                type: '',
                header: '',
                to_fix: ''
            },
            modalInfo: defaultModalInfo,
            savedAnalysisNames: this.props.savedAnalysisNames,
            loading: false,
            columnWidthsArray: defaultColumnWidthsArray,
            isLocalDeployment: props.isLocalDeployment,
            currOpenTaskpane: currOpenTaskpane,
            mitoContainerRef: undefined,
            stepSummaryList: this.props.stepSummaryList,
            currStepIdx: this.props.currStepIdx,
            selectedColumnControlPanelTab: ControlPanelTab.FilterSort,
            tourJSON: props.tourJSON,
            highlightPivotTableButton: false,
            highlightAddColButton: false,
            // We default the current path parts to the current folder, 
            // and on the backend make sure to handle this case in our
            // path parsing
            currPathParts: ['.'],
            lastGraphParams: {},
        };
        this.cellFocused = this.cellFocused.bind(this);
        this.setEditingMode = this.setEditingMode.bind(this);
        this.setEditingFormula = this.setEditingFormula.bind(this);
        this.setModal = this.setModal.bind(this);
        this.getCurrentModalComponent = this.getCurrentModalComponent.bind(this);
        this.setSelectedSheetIndex = this.setSelectedSheetIndex.bind(this);
        this.turnOnCellEditor = this.turnOnCellEditor.bind(this);
        this.setCursorIndex = this.setCursorIndex.bind(this);
        this.columnMoved = this.columnMoved.bind(this);
        this.columnDragStopped = this.columnDragStopped.bind(this);
        this.setColumnWidth = this.setColumnWidth.bind(this);
        this.setCurrOpenTaskpane = this.setCurrOpenTaskpane.bind(this);
        this.getCurrOpenTaskpane = this.getCurrOpenTaskpane.bind(this);
        this.closeOpenEditingPopups = this.closeOpenEditingPopups.bind(this);
        this.setSelectedColumn = this.setSelectedColumn.bind(this);
        this.refreshSelectedCell = this.refreshSelectedCell.bind(this);
        this.setSelectedColumnControlPanelTab = this.setSelectedColumnControlPanelTab.bind(this);
        this.setHighlightPivotTableButton = this.setHighlightPivotTableButton.bind(this);
        this.setHighlightAddColButton = this.setHighlightAddColButton.bind(this);
        this.displayTour = this.displayTour.bind(this);
        this.setCurrPathParts = this.setCurrPathParts.bind(this);
        this.setColumnRenameOpenOrClosed = this.setColumnRenameOpenOrClosed.bind(this);
        this.setLastGraphParams = this.setLastGraphParams.bind(this);
    }
    /*
        The componentDidUpdate hook is responsible for maintaining consistency in the state
        variables of this prop in the case of updates to the sheet data. NOTE: this consistency
        should enforce that any column you index into should be defined for all sheets + all columns
        (e.g. this.state.columnFilters should be defined everywhere). If this is not the case, then it MUST
        be enforced in the type system!

        For example, if a sheet is deleted, then selectedSheetIndex in that state should not
        be set equal to the old index of that sheet.

        Furthermore, it makes sure the selected column is actually a column that exists in
        the dataset.

        Furthermore, if the sheet being switched to is a sheet that was created through a
        pivot, then it opens the pivot interface.

        See more here: https://reactjs.org/docs/react-component.html#componentdidupdate

        NOTE: careful on calling this.setState in this function, as repeated calls cause
        the sheet to crash (as this recurses infinitely).

        NOTE: this function will fail if it is called on a dataframe with mulitple columns with
        the same name. This is an invariant we currently expect to be the case.
    */
    async componentDidUpdate(prevProps, prevState) {
        const newSheetJSON = this.state.sheetJSONArray;
        let newSelectedSheetIndex = this.state.selectedSheetIndex;
        // If the selectedSheetIndex no longer exists, because of an undo, don't select that one!
        // Or, if there was an a new sheet creation, then default to the last sheet
        if (newSelectedSheetIndex >= newSheetJSON.length || prevState.sheetJSONArray.length < newSheetJSON.length) {
            newSelectedSheetIndex = newSheetJSON.length - 1;
        }
        if (newSelectedSheetIndex != this.state.selectedSheetIndex) {
            this.setState({
                selectedSheetIndex: newSelectedSheetIndex
            });
        }
        // Check if the newly switched to sheet is a pivoted sheet, and if there isn't
        // a taskpane open currently, and open the pivot table to edit in this case
        if (newSelectedSheetIndex !== prevState.selectedSheetIndex &&
            this.state.dfSources[newSelectedSheetIndex] === DFSource.Pivoted &&
            this.state.currOpenTaskpane.type === TaskpaneType.NONE) {
            const existingPivotParams = await this.props.mitoAPI.getPivotParams(newSelectedSheetIndex);
            if (existingPivotParams !== undefined) {
                this.setState({
                    currOpenTaskpane: {
                        type: TaskpaneType.PIVOT,
                        destinationSheetIndex: newSelectedSheetIndex,
                        existingPivotParams: existingPivotParams
                    }
                });
            }
        }
        // Make sure the column widths are set correctly, which they may not be if the columns in sheet
        // have changed, or a sheet has been added or subtracted
        const columnWidthsArrayResult = getColumnWidthsArray(this.state.sheetJSONArray, this.state.columnWidthsArray);
        // Check if we need to actually update the column widths (sheet number change, any column change)
        if (columnWidthsArrayResult.changed) {
            this.setState({
                columnWidthsArray: columnWidthsArrayResult.columnWidthsArray
            });
        }
        // Make sure the selected column is valid and actually from the selected sheet
        if (this.state.sheetJSONArray[newSelectedSheetIndex] !== undefined
            && !this.state.sheetJSONArray[newSelectedSheetIndex].columnIDs.includes(this.state.selectedColumnID)
            // NOTE: this last condition checks if there is no column to select
            && !(this.state.sheetJSONArray[newSelectedSheetIndex].columnIDs.length === 0 && this.state.selectedColumnID === '')) {
            const newSelectedColumnID = getNewSelectedColumn(this.state.sheetJSONArray[newSelectedSheetIndex].columnIDs, prevState.sheetJSONArray[newSelectedSheetIndex] !== undefined ?
                {
                    oldColumnHeaders: prevState.sheetJSONArray[newSelectedSheetIndex].columnIDs,
                    oldSelectedColumnHeader: prevState.selectedColumnID
                }
                : undefined);
            this.setSelectedColumn(newSelectedColumnID);
        }
    }
    /*
        Called by the cell editor so that the column the user clicks on can be appended
        to the correct location in the formula
    */
    setCursorIndex(index) {
        this.setState(prevState => {
            return {
                editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { formulaCursorIndex: index })
            };
        });
    }
    /*
        This function is responsible for turning cell editing mode on and off
        by setting the state of editingColumnID, and editingRowIndex.

        If you call this function with newEditingMode===CellEditingMode.COLUMN_EDITING or newEditingMode===CellEditingMode.SINGLE_CELL_EDITING,
        then the passed column and rowIndex should be the cell you want to start editing.

        If you call this function with newEditingMode===CellEditingMode.OFF, then editing will be stopped, and the
        passed column and rowIndex will be focused on.
    */
    async setEditingMode(newEditingMode, columnID, rowIndex) {
        // If we are turning editing mode on and editing mode is currently off
        if ((newEditingMode === CellEditingMode.COLUMN_EDITING || newEditingMode === CellEditingMode.SINGLE_CELL_EDITING)
            && this.state.editingModeState.editingMode === CellEditingMode.OFF) {
            // we close the taskpane if its open and its a editing taskpane
            this.closeOpenEditingPopups();
            /*
                This runs (and turns on editing mode in COLUMN_EDITING state) when we're not in editing mode when:
                1. The user double clicks on a cell
                2. The user presses enter on a cell.
                3. The user types any character on a cell.
            */
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingSheetIndex: this.state.selectedSheetIndex, editingMode: newEditingMode, editingColumnID: columnID, editingRowIndex: rowIndex })
                };
            });
        }
        else if (newEditingMode === CellEditingMode.OFF) {
            /*
                If Cell Editing Mode is not on, don't do anything.

                We rely on this to not get stuck in a loop of submitting invalid formulas to the backend.
                Without this check, when a user submits an invalid formula, it generates an error, but
                the error modal also turns off cell editing mode, so the formula gets submitted again continuously.
            */
            if (this.state.editingModeState.editingMode === CellEditingMode.COLUMN_EDITING ||
                this.state.editingModeState.editingMode === CellEditingMode.SINGLE_CELL_EDITING) {
                /*
                    We turn off cell-editing mode and select the given cell.

                    This handles some of the many ways cell editing can be stopped
                    explicitly (e.g. ENTER), as we want only sometimes want ENTER to stop
                    editing (in other cases, we want it to select a suggestion!).

                    To see a list of events that stop editing, see:
                    https://www.ag-grid.com/javascript-grid-cell-editing/#stop-end-editing
                */
                // We save this variable so we can reference it in the callback after we update the state.
                const typeOfEditingMode = this.state.editingModeState.editingMode;
                this.setState(prevState => {
                    return {
                        editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingMode: CellEditingMode.OFF })
                    };
                }, async () => {
                    var _a, _b, _c, _d;
                    /*
                        We then get ready to submit the formula to the backend by getting the formula to submit
                        
                        We don't use the column and rowIndex passed to the function because they are only accurate
                        when turning on cellEditingMode.
                    */
                    const selectedColumnID = this.state.editingModeState.editingColumnID;
                    const selectedRowIndex = this.state.editingModeState.editingRowIndex;
                    const newFormulaOrValue = this.state.editingModeState.lastFormula;
                    // We only submit if the newSpreadsheetColumnFormula is defined to avoid crashing the sheet. 
                    if (selectedColumnID !== '' && newFormulaOrValue !== undefined) {
                        if (typeOfEditingMode === CellEditingMode.COLUMN_EDITING) {
                            await this.props.mitoAPI.sendSetColumnFormulaEditMessage(this.state.selectedSheetIndex, selectedColumnID, newFormulaOrValue);
                        }
                        else {
                            // Get the actual index in the dataframe, to edit the right item
                            const dataFrameIndex = this.state.sheetJSONArray[this.state.selectedSheetIndex].index[selectedRowIndex];
                            await this.props.mitoAPI.sendSetCellValueMessage(this.state.selectedSheetIndex, selectedColumnID, dataFrameIndex, newFormulaOrValue);
                        }
                    }
                    // We actually stop the grid from editing in this case, and set cell focus
                    // as stopping editing focuses on nothing by default.
                    (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.stopEditing();
                    (_d = (_c = window.gridApiMap) === null || _c === void 0 ? void 0 : _c.get(this.props.model_id)) === null || _d === void 0 ? void 0 : _d.setFocusedCell(rowIndex, columnID);
                });
            }
        }
    }
    /*
        This function is called by the cell editor to update the formula of the editing cell
        in the mito state, so that when cellFocused starts editing mode again, it can
        pass in the starting formula.

        We save the formula in the state because:
            1) when the user clicks on another cell, it closes the cell editor.
            We need to store the formula in state so when we put the editor
            back in cell editing mode in the CellFocussedEvent,
            we're able to start the cell editor with the correct formula.
            
            Note: we don't want to send the intermediate formula to the parser

            2) we need to display the formula that is in the editor in the formula bar as well.
    */
    setEditingFormula(formula) {
        this.setState(prevState => {
            return {
                editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { lastFormula: formula }),
                formulaBarValue: formula,
            };
        });
    }
    turnOnCellEditor() {
        var _a, _b, _c, _d;
        const editingModeParams = {
            rowIndex: this.state.editingModeState.editingRowIndex,
            colKey: this.state.editingModeState.editingColumnID,
        };
        // Scroll to make sure this editor is visible (but only scroll just enough)
        (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.ensureColumnVisible(editingModeParams.colKey);
        // TODO: we might want to ensureIndexVisible as well, but users always edit at the top
        // turn the editing cell's cell editor back on!
        (_d = (_c = window.gridApiMap) === null || _c === void 0 ? void 0 : _c.get(this.props.model_id)) === null || _d === void 0 ? void 0 : _d.startEditingCell(editingModeParams);
    }
    /*
        Occurs when a cell is clicked. If we are currently editing a cell, we append the clicked
        column to the currently-edited cell. Otherwise, we select the clicked cell.
    */
    cellFocused(event) {
        /*
            We avoid cell focused throwing an error when switching sheets by making sure the
            event is defined, and just returning. Not sure why this occurs!

            Additionally, turn of cell editing mode & reset the formulaBarValue. when the user switches
            sheets we reset the editing cell to clear the formula bar
        */
        var _a;
        const columnID = (_a = event.column) === null || _a === void 0 ? void 0 : _a.getColId();
        if (!columnID) {
            void this.setEditingMode(CellEditingMode.OFF, "", -1);
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { editingColumnID: '', editingRowIndex: -1 }),
                    formulaBarValue: '',
                };
            });
            return;
        }
        // If we're in cell editing mode, turn cell editor back on
        if (this.state.editingModeState.editingMode === CellEditingMode.COLUMN_EDITING) {
            /*
                If the column we focussed on is the editing column or the index column, don't append to the formula
            */
            if (columnID === this.state.editingModeState.editingColumnID || columnID === MITO_INDEX) {
                this.turnOnCellEditor();
                return;
            }
            /*
            If we're in editing mode & the cell focused on is not in the column we're editing
            or the index column, append the clicked column to the editing formula; which
            we have to look up by ID
            */
            const columnHeader = this.state.columnIDsMapArray[this.state.selectedSheetIndex][columnID].toString();
            // Add the column reference to the formula where the user's cursor is in the input field
            const formulaCursorIndex = this.state.editingModeState.formulaCursorIndex;
            const newFormula = this.state.formulaBarValue.slice(0, formulaCursorIndex) + columnHeader + this.state.formulaBarValue.slice(formulaCursorIndex);
            this.setState(prevState => {
                return {
                    editingModeState: Object.assign(Object.assign({}, prevState.editingModeState), { lastFormula: newFormula, formulaCursorIndex: formulaCursorIndex + columnHeader.length // update the formula cursor index to include appended characters
                     }),
                    formulaBarValue: newFormula,
                };
            }, () => {
                this.turnOnCellEditor();
            });
        }
        else if (this.state.editingModeState.editingMode === CellEditingMode.SINGLE_CELL_EDITING) {
            const rowIndex = event.rowIndex;
            if (columnID === this.state.editingModeState.editingColumnID && rowIndex === this.state.editingModeState.editingRowIndex) {
                // If the column focused on is the cell we are editing (this happens when you press enter to begin single cell editing mode)
                // then don't do anything.
                return;
            }
            else {
                // Otherwise, if we focussed on a different cell, then turn off editing mode. 
                void this.setEditingMode(CellEditingMode.OFF, '', -1);
            }
        }
        else {
            // If we're not in editing mode, then we update the formula bar only
            // If the column is the index column, then we reset the selected cell state
            if (columnID === MITO_INDEX) {
                this.setState({
                    selectedRowIndex: 0,
                    formulaBarValue: ''
                });
                this.setSelectedColumn('');
                return;
            }
            // Otherwise, get the cell's formula to display
            const columnIndex = this.state.sheetJSONArray[this.state.selectedSheetIndex].columnIDs.indexOf(columnID);
            const rowIndex = event.rowIndex;
            const columnFormula = this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex][columnID];
            let formulaBarValue = this.state.formulaBarValue;
            if (columnFormula === '') {
                // If there no formula, we just display the value
                formulaBarValue = this.state.sheetJSONArray[this.state.selectedSheetIndex].data[rowIndex][columnIndex];
            }
            else {
                // Otherwise, we display the formula that is in the column
                formulaBarValue = columnFormula;
                if (this.state.selectedSheetIndex === this.state.editingModeState.editingSheetIndex && columnID === this.state.editingModeState.editingColumnID) {
                    // The last formula is always equal to the _last_ edit made to a formula
                    // and so we take it so we keep an error formula
                    formulaBarValue = this.state.editingModeState.lastFormula;
                }
            }
            this.setState({
                selectedRowIndex: rowIndex,
                formulaBarValue: formulaBarValue
            });
            this.setSelectedColumn(columnID);
        }
    }
    getCurrentModalComponent() {
        // Returns the JSX.element that is currently, open, and is used
        // in render to display this modal
        switch (this.state.modalInfo.type) {
            case ModalEnum.None: return React.createElement("div", null);
            case ModalEnum.Error: return (React.createElement(ErrorModal, { errorJSON: this.state.errorJSON, setModal: this.setModal }));
            case ModalEnum.ClearAnalysis: return (React.createElement(ClearAnalysisModal, { setModal: this.setModal, mitoAPI: this.props.mitoAPI }));
            case ModalEnum.SaveAnalysisOverwrite: return (React.createElement(SaveAnalysisOverwriteModal, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, setModal: this.setModal, savedAnalysisNames: this.state.savedAnalysisNames, saveAnalysisName: this.state.modalInfo.saveAnalysisName, mitoAPI: this.props.mitoAPI }));
            case ModalEnum.ReplayImport: return (React.createElement(ReplayImportModal, { setModal: this.setModal, dfNames: this.state.dfNames, api: this.props.mitoAPI, analysisName: this.state.modalInfo.analysisName, importSummaries: this.state.modalInfo.importSummary, mitoAPI: this.props.mitoAPI }));
            case ModalEnum.SignUp: return (React.createElement(SignUpModal, { setModal: this.setModal, mitoAPI: this.props.mitoAPI }));
            case ModalEnum.Upgrade: return (React.createElement(UpgradeModal, { setModal: this.setModal, mitoAPI: this.props.mitoAPI }));
        }
    }
    setModal(modalInfo) {
        // Sets the current modal. We _always_ turn off cell editing mode here, just
        // in case the user opens/closes a modal in cell editing mode.
        void this.setEditingMode(CellEditingMode.OFF, "", -1);
        // unless opening an error modal or closing a modal, close open editing taskpanes
        if (modalInfo.type !== ModalEnum.Error && modalInfo.type !== ModalEnum.None) {
            this.closeOpenEditingPopups();
        }
        // And then we actually update the modalInfo
        this.setState({ modalInfo: modalInfo });
    }
    /*
        Set the selected tab on the column control panel,
        specifically to either filter and sort or the summary statistics
    */
    setSelectedColumnControlPanelTab(tab) {
        this.setState({
            selectedColumnControlPanelTab: tab
        });
        if (tab == ControlPanelTab.FilterSort) {
            void this.props.mitoAPI.sendLogMessage('clicked_sort_and_filter_tab', { 'column_id': this.state.selectedColumnID });
        }
        else if (tab == ControlPanelTab.UniqueValues) {
            void this.props.mitoAPI.sendLogMessage('clicked_unique_values_tab', { 'column_id': this.state.selectedColumnID });
        }
        else {
            void this.props.mitoAPI.sendLogMessage('clicked_summary_statistics_tab', { 'column_id': this.state.selectedColumnID });
        }
    }
    setColumnRenameOpenOrClosed(isRename) {
        if (this.state.currOpenTaskpane.type === TaskpaneType.CONTROL_PANEL) {
            this.setState({ currOpenTaskpane: {
                    type: TaskpaneType.CONTROL_PANEL,
                    openEditingColumnHeader: isRename
                } });
        }
    }
    /*
        Sets the current path parts in the mito state, so that
        the import taskpane can open to the same place next time
        it is used
    */
    setCurrPathParts(pathParts) {
        this.setState({ currPathParts: pathParts });
    }
    /*
        Sets the current path parts in the mito state, so that
        the import taskpane can open to the same place next time
        it is used
    */
    setLastGraphParams(sheetIndex, graphParams) {
        this.setState((prevState) => {
            return Object.assign(Object.assign({}, prevState), { lastGraphParams: Object.assign(Object.assign({}, prevState.lastGraphParams), { [sheetIndex]: graphParams }) });
        });
    }
    /*
        Returns the JSX.Element of the taskpane that should be currently
        displayed, stored in this.state.currOpenTaskpane.

        If this.state.currOpenTaskpane is undefined, then returns no taskpane
        at all.
    */
    getCurrOpenTaskpane() {
        switch (this.state.currOpenTaskpane.type) {
            case TaskpaneType.NONE: return (React.createElement(Fragment, null));
            case TaskpaneType.PIVOT: return (React.createElement(PivotTaskpane, { dfNames: this.state.dfNames, sheetJSONArray: this.state.sheetJSONArray, columnIDsMapArray: this.state.columnIDsMapArray, mitoAPI: this.props.mitoAPI, selectedSheetIndex: this.state.selectedSheetIndex, lastStepIndex: this.state.stepSummaryList[this.state.stepSummaryList.length - 1].step_idx, setCurrOpenTaskpane: this.setCurrOpenTaskpane, destinationSheetIndex: this.state.currOpenTaskpane.destinationSheetIndex, existingPivotParams: this.state.currOpenTaskpane.existingPivotParams }));
            case TaskpaneType.MERGE: return (React.createElement(MergeTaskpane, { dfNames: this.state.dfNames, columnIDsMapArray: this.state.columnIDsMapArray, selectedSheetIndex: this.state.selectedSheetIndex, sheetJSONArray: this.state.sheetJSONArray, setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI }));
            case TaskpaneType.DOWNLOAD: return (React.createElement(DownloadTaskpane, { dfName: this.state.dfNames[this.state.selectedSheetIndex], selectedSheetIndex: this.state.selectedSheetIndex, setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI }));
            case TaskpaneType.SAVE: return (React.createElement(SaveTaskpane, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, setModal: this.setModal, savedAnalysisNames: this.state.savedAnalysisNames, saveAnalysisName: this.state.currOpenTaskpane.saveAnalysisName, mitoAPI: this.props.mitoAPI }));
            case TaskpaneType.IMPORT: return (React.createElement(ImportTaskpane, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI, currPathParts: this.state.currPathParts, setCurrPathParts: this.setCurrPathParts }));
            case TaskpaneType.STEPS: return (React.createElement(StepsTaskpane, { stepSummaryList: this.state.stepSummaryList, setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI, currStepIdx: this.state.currStepIdx }));
            case TaskpaneType.DOCUMENTATION: return (React.createElement(DocumentationTaskpane, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI }));
            case TaskpaneType.FEEDBACK: return (React.createElement(FeedbackTaskpane, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI, openLocation: this.state.currOpenTaskpane.openLocation, numFeedbacks: this.props.numFeedbacks, numUsages: this.props.numUsages }));
            case TaskpaneType.FEEDBACK_THANKS: return (React.createElement(FeedbackThanksTaskpane, { setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI }));
            case TaskpaneType.CONTROL_PANEL:
                /*
                    We define default values here so when we rename the column header through the control panel,
                    we don't have to worry about a sheet-crashing race condition where the Mito state is not yet updated to reflect the
                    updated backend state.
                */
                // eslint-disable-next-line no-case-declarations
                let filters = [];
                // eslint-disable-next-line no-case-declarations
                let operator = 'And';
                // eslint-disable-next-line no-case-declarations
                let columnMitoType = "string_series" /* STRING_SERIES */;
                // eslint-disable-next-line no-case-declarations
                let columnFormula = '';
                if (this.state.columnFiltersArray[this.state.selectedSheetIndex] !== undefined && this.state.columnFiltersArray[this.state.selectedSheetIndex][this.state.selectedColumnID] !== undefined) {
                    filters = this.state.columnFiltersArray[this.state.selectedSheetIndex][this.state.selectedColumnID].filters;
                    operator = this.state.columnFiltersArray[this.state.selectedSheetIndex][this.state.selectedColumnID].operator;
                    columnMitoType = this.state.columnTypeJSONArray[this.state.selectedSheetIndex][this.state.selectedColumnID];
                }
                if (this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex] !== undefined) {
                    columnFormula = this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex][this.state.selectedColumnID];
                }
                return (React.createElement(ControlPanelTaskpane
                // set the columnHeader as the key so that the taskpane updates when it is switched
                // TODO: figure out why we need this, if the other variables update?
                , { 
                    // set the columnHeader as the key so that the taskpane updates when it is switched
                    // TODO: figure out why we need this, if the other variables update?
                    key: this.state.selectedColumnID + this.state.currOpenTaskpane.openEditingColumnHeader, selectedSheetIndex: this.state.selectedSheetIndex, columnIDsMapArray: this.state.columnIDsMapArray, columnID: this.state.selectedColumnID, columnFormula: columnFormula, openEditingColumnHeader: this.state.currOpenTaskpane.openEditingColumnHeader, columnMitoType: columnMitoType, operator: operator, filters: filters, setCurrOpenTaskpane: this.setCurrOpenTaskpane, setColumnRenameOpenOrClosed: this.setColumnRenameOpenOrClosed, mitoAPI: this.props.mitoAPI, setSelectedColumnControlPanelTab: this.setSelectedColumnControlPanelTab, tab: this.state.selectedColumnControlPanelTab, lastStepIndex: this.state.stepSummaryList[this.state.stepSummaryList.length - 1].step_idx }));
            case TaskpaneType.REPLAY_ANALYSIS:
                return (React.createElement(ReplayAnalysisTaskpane, { savedAnalysisNames: this.state.savedAnalysisNames, mitoAPI: this.props.mitoAPI, setCurrOpenTaskpane: this.setCurrOpenTaskpane, setModal: this.setModal }));
            case TaskpaneType.GRAPH:
                return (React.createElement(GraphSidebar, { graphSidebarSheet: this.state.currOpenTaskpane.graphSidebarSheet, dfNames: this.state.dfNames, columnIDsMapArray: this.state.columnIDsMapArray, sheetJSONArray: this.state.sheetJSONArray, columnMitoTypes: this.state.columnTypeJSONArray[this.state.selectedSheetIndex], mitoAPI: this.props.mitoAPI, setSelectedSheetIndex: this.setSelectedSheetIndex, setCurrOpenTaskpane: this.setCurrOpenTaskpane, setSelectedColumn: this.setSelectedColumn, model_id: this.props.model_id, lastGraphParams: this.state.lastGraphParams, setLastGraphParams: this.setLastGraphParams }));
        }
    }
    /*
        Changes what taskpane is currently displayed. Set to undefined
        to close the open taskpane.
    */
    setCurrOpenTaskpane(newTaskpaneInfo) {
        // Log this new change, including some metadata about
        // the current state of the sheet
        const logParams = {
            new_taskpane: newTaskpaneInfo.type,
            sheet_index: this.state.selectedSheetIndex,
            column_id: this.state.selectedColumnID
        };
        void this.props.mitoAPI.sendLogMessage('set_curr_open_taskpane', logParams);
        this.setState({
            currOpenTaskpane: newTaskpaneInfo,
        });
    }
    /*
        Closes any open editing popups, which includes:
        1. Any open sheet tab actions
        2. The taskpane, if it is an EDITING_TASKPANE

        Allows you to optionally specify a list of taskpanes to keep open if they
        are currently open, which is useful for undo, for example,
        when editing a pivot table and pressing undo
    */
    closeOpenEditingPopups(taskpanesToKeepIfOpen) {
        // Close the taskpane if it is an editing taskpane, and it is not in the list of taskpanesToKeepIfOpen
        if (EDITING_TASKPANES.includes(this.state.currOpenTaskpane.type) && (taskpanesToKeepIfOpen === undefined || !taskpanesToKeepIfOpen.includes(this.state.currOpenTaskpane.type))) {
            this.setState({
                currOpenTaskpane: { type: TaskpaneType.NONE }
            });
        }
    }
    setSelectedSheetIndex(newIndex) {
        // We turn off editing mode, if it is on, because we moved to a new sheet
        void this.setEditingMode(CellEditingMode.OFF, "", -1);
        // We also make sure we don't switch to a sheet that doesn't exist
        const inBoundsNewIndex = Math.min(newIndex, this.state.sheetJSONArray.length - 1);
        // We use the backend to log that we switched sheets
        // NOTE: we don't await, because we don't care when this happens
        void this.props.mitoAPI.sendLogMessage('switch_sheet_log_event', {
            'sheet_index': newIndex
        });
        // And then we update the selected index
        this.setState({
            selectedSheetIndex: inBoundsNewIndex
        }, () => {
            // We set the selected column back to the first column of the newly selected sheet
            this.setSelectedColumn(this.state.sheetJSONArray[inBoundsNewIndex].columnIDs[0]);
        });
    }
    /*
        The ColumnMovedEvent gets fired when a user drags a column to change its location.
        While dragging the column from one position to another, this event gets fired multiple times,
        for both the column that is moved but also sometimes for undefined columns.

        Thus, when the column is moving, and it is defined, we set the movingColumnParams
        so we can send a reorder event to the backend when it stops moving. See columnDragStopped
    */
    columnMoved(event) {
        var _a;
        const colId = (_a = event.column) === null || _a === void 0 ? void 0 : _a.getColId();
        const newIndex = event.toIndex;
        if (colId && newIndex) {
            this.setState({
                movingColumnParams: {
                    column: colId,
                    newIndex: newIndex
                }
            });
        }
    }
    // When the column stops moving, send the column reorder event to the backend 
    async columnDragStopped() {
        /*
            The DragStoppedEvent gets fired when the user changes the width of the column and when
            the user reorders the column. We only want to send a column_reorder_event when the user is
            reordering the column, so we check if the user is moving a column by checking this.state.columnMovedState.column
        */
        if (this.state.movingColumnParams != undefined) {
            // If the column did not change locations, don't send a column_reorder_event
            if (this.state.sheetJSONArray[this.state.selectedSheetIndex].columnIDs[this.state.movingColumnParams.newIndex - 1] === this.state.movingColumnParams.column) {
                return;
            }
            void this.props.mitoAPI.sendReorderColumnMessage(this.state.selectedSheetIndex, this.state.movingColumnParams.column, this.state.movingColumnParams.newIndex - 1);
            // Reset the columnMovedState so a column resizing doesn't cause the column_reorder_event to get sent
            this.setState({
                movingColumnParams: undefined
            });
        }
    }
    setColumnWidth(sheetIndex, columnHeader, columnWidth) {
        this.setState(prevState => {
            const newColumnsWidthsArray = [...prevState.columnWidthsArray];
            newColumnsWidthsArray[sheetIndex][columnHeader] = columnWidth;
            return Object.assign(Object.assign({}, prevState), { columnWidthsArray: newColumnsWidthsArray });
        });
    }
    /*
        Sets the selectedColumn in the Mito state and tells Ag-grid
        to redraw the old selected column and new selected column to keep
        the selected column highlighting in sync with state

        Furthermore, if there is currently a column control panel open, it
        refreshes the column that is opened in the column control panel.

        Note: the selectedColumn state should not be updated without this function
    */
    setSelectedColumn(columnId) {
        let oldSelectedColumnId = '';
        this.setState((prevState) => {
            oldSelectedColumnId = prevState.selectedColumnID;
            return {
                selectedColumnID: columnId
            };
        }, () => {
            var _a, _b;
            // force the columns to refresh.  
            const params = {
                force: true,
                suppressFlash: true,
                columns: [oldSelectedColumnId, columnId],
            };
            (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.refreshCells(params);
        });
    }
    /*
        This function is used to refresh the selected cell. It is called by the
        handleMessage function in the case where the received message is
        a update_sheet event.
        
        It is not called when the received message is edit_error so that we preserve
        the invalid formula in the cell, making it easy to correct the error.

        We use this function to avoid resubmitted formulas from getting stuck
        in the rendered cells.

        If the selected column is a data column, then we return
    */
    refreshSelectedCell() {
        var _a, _b;
        const selectedColumnID = this.state.selectedColumnID;
        // If there is no selectedColumn, just return.
        if (selectedColumnID === '') {
            return;
        }
        // If the selectedSheetIndex does not exist, just return.
        // This occurs after a dataframe delete.
        if (this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex] === undefined) {
            return;
        }
        // If the selectedColumn does not exist in the columnSpreadsheetCodeJSON, just return.
        // This occurs after a column rename
        if (!(selectedColumnID in this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex])) {
            return;
        }
        // If the column is not editable (aka it's a data column), just return. 
        if (this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex][selectedColumnID] === '') {
            return;
        }
        // Get the selectedRowNode to access the cell we are refreshing
        const selectedRowNode = (_b = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(this.props.model_id)) === null || _b === void 0 ? void 0 : _b.getDisplayedRowAtIndex(this.state.selectedRowIndex);
        const sheetJSON = this.state.sheetJSONArray[this.state.selectedSheetIndex];
        // If the sheetJSON or selectedRowNode is undefined, just return
        if (sheetJSON === undefined || selectedRowNode === undefined) {
            return;
        }
        const selectedColumnIndex = sheetJSON.columnIDs.indexOf(selectedColumnID);
        const newValue = sheetJSON.data[this.state.selectedRowIndex][selectedColumnIndex];
        // Set the cell's value
        selectedRowNode === null || selectedRowNode === void 0 ? void 0 : selectedRowNode.setDataValue(this.state.selectedColumnID, newValue);
    }
    /*
        We currrently send all users through the intro tour,
        as well as the explore data tour.
    */
    displayTour() {
        const toursToDisplay = [];
        // If the user has either no or tutorial data in the tool, don't display the tour
        if (this.state.tourJSON.typeOfDataInTool === DataTypeInMito.NONE || this.state.tourJSON.typeOfDataInTool === DataTypeInMito.TUTORIAL) {
            return toursToDisplay;
        }
        // If the user has not taken the Intro Tour yet, return Intro Tour
        if (!this.state.tourJSON.receivedTours.includes(TourName.INTRO)) {
            toursToDisplay.push(TourName.INTRO);
        }
        // If the user hasn't taken the explore data tour, we return that also
        if (!this.state.tourJSON.receivedTours.includes(TourName.EXPLORE_DATA)) {
            toursToDisplay.push(TourName.EXPLORE_DATA);
        }
        return toursToDisplay;
    }
    /* Used the by the tour to highlight the pivot table button */
    setHighlightPivotTableButton(highlight) {
        this.setState({
            highlightPivotTableButton: highlight
        });
    }
    /* Used the by the tour to highlight the Add Col button */
    setHighlightAddColButton(highlight) {
        this.setState({
            highlightAddColButton: highlight
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    render() {
        /*
            We detect whether the taskpane is open in wide mode, narrow mode, or not open at all. We then
            set the class of the div containing the Mitosheet and Formula bar, as well as the taskpane div accordingly.
            The class sets the width of the sheet.
        */
        let mitoFormulaBarAndMitosheetClass = '';
        let mitoDefaultTaskpaneClass = 'mito-default-taskpane';
        if (WIDE_TASKPANES.includes(this.state.currOpenTaskpane.type)) {
            mitoFormulaBarAndMitosheetClass = 'mito-formula-bar-and-mitosheet-div-wide-taskpane-open';
            mitoDefaultTaskpaneClass = 'mito-default-wide-taskpane-open';
        }
        else if (this.state.currOpenTaskpane.type !== TaskpaneType.NONE) {
            mitoFormulaBarAndMitosheetClass = 'mito-formula-bar-and-mitosheet-div-narrow-taskpane-open';
            mitoDefaultTaskpaneClass = 'mito-default-narrow-taskpane-open';
        }
        return (React.createElement("div", { className: "mito-container", "data-jp-suppress-context-menu": true, ref: (ref) => {
                // NOTE: it is crucial we only set the ref it is not already defined, as we
                // will otherwise enter into an infinite loop
                if (!this.state.mitoContainerRef) {
                    this.setState({ mitoContainerRef: ref });
                }
            } },
            React.createElement(Toolbar, { mitoContainerRef: this.state.mitoContainerRef, sheetJSON: this.state.sheetJSONArray[this.state.selectedSheetIndex], columnIDMap: this.state.columnIDsMapArray[this.state.selectedSheetIndex], dfSources: this.state.dfSources, selectedSheetIndex: this.state.selectedSheetIndex, setEditingMode: this.setEditingMode, setModal: this.setModal, model_id: this.props.model_id, selectedColumnID: this.state.selectedColumnID, setCurrOpenTaskpane: this.setCurrOpenTaskpane, mitoAPI: this.props.mitoAPI, closeOpenEditingPopups: this.closeOpenEditingPopups, currStepIdx: this.state.currStepIdx, lastStepIndex: this.state.stepSummaryList[this.state.stepSummaryList.length - 1].step_idx, highlightPivotTableButton: this.state.highlightPivotTableButton, highlightAddColButton: this.state.highlightAddColButton }),
            React.createElement("div", { className: "mito-main-sheet-div" },
                React.createElement("div", { className: 'mito-formula-bar-and-mitosheet-div ' + mitoFormulaBarAndMitosheetClass },
                    React.createElement(FormulaBar, { selectedColumnHeader: this.state.columnIDsMapArray[this.state.selectedSheetIndex] !== undefined ? this.state.columnIDsMapArray[this.state.selectedSheetIndex][this.state.selectedColumnID] : '', formulaBarValue: this.state.formulaBarValue }),
                    React.createElement(MitoSheet, { selectedSheetIndex: this.state.selectedSheetIndex, columnIDsMap: this.state.columnIDsMapArray[this.state.selectedSheetIndex] || {}, columnMitoTypes: this.state.columnTypeJSONArray[this.state.selectedSheetIndex], formulaBarValue: this.state.formulaBarValue, editingColumnID: this.state.editingModeState.editingColumnID, selectedColumnID: this.state.selectedColumnID, columnSpreadsheetCodeJSON: this.state.columnSpreadsheetCodeJSONArray[this.state.selectedSheetIndex], sheetJSON: this.state.sheetJSONArray[this.state.selectedSheetIndex], columnFiltersJSON: this.state.columnFiltersArray[this.state.selectedSheetIndex], setEditingMode: this.setEditingMode, setEditingFormula: this.setEditingFormula, setCursorIndex: this.setCursorIndex, cursorIndex: this.state.editingModeState.formulaCursorIndex, cellFocused: this.cellFocused, columnMoved: this.columnMoved, columnDragStopped: this.columnDragStopped, model_id: this.props.model_id, columnWidths: this.state.columnWidthsArray[this.state.selectedSheetIndex], setColumnWidth: this.setColumnWidth, setCurrOpenTaskpane: this.setCurrOpenTaskpane, setSelectedColumn: this.setSelectedColumn, setSelectedColumnControlPanelTab: this.setSelectedColumnControlPanelTab })),
                this.state.currOpenTaskpane.type !== TaskpaneType.NONE &&
                    React.createElement("div", { className: mitoDefaultTaskpaneClass }, this.getCurrOpenTaskpane())),
            this.displayTour().length !== 0 && this.state.modalInfo.type !== ModalEnum.SignUp &&
                React.createElement(Tour, { sheetJSON: this.state.sheetJSONArray[this.state.selectedSheetIndex], setHighlightPivotTableButton: this.setHighlightPivotTableButton, setHighlightAddColButton: this.setHighlightAddColButton, tourNames: this.displayTour(), mitoAPI: this.props.mitoAPI }),
            React.createElement(Footer, { dfNames: this.state.dfNames, sheetShapeArray: this.state.sheetShapeArray, selectedSheetIndex: this.state.selectedSheetIndex, setSelectedSheetIndex: this.setSelectedSheetIndex, mitoAPI: this.props.mitoAPI, closeOpenEditingPopups: this.closeOpenEditingPopups, setCurrOpenTaskpane: this.setCurrOpenTaskpane }),
            this.getCurrentModalComponent(),
            this.state.loading && React.createElement(LoadingIndicator, null),
            this.state.currStepIdx !== this.state.stepSummaryList[this.state.stepSummaryList.length - 1].step_idx &&
                React.createElement(CatchUpPopup, { fastForward: () => {
                        void this.props.mitoAPI.checkoutStepByIndex(this.state.stepSummaryList[this.state.stepSummaryList.length - 1].step_idx);
                    } })));
    }
}
export default Mito;
//# sourceMappingURL=Mito.js.map