// Copyright (c) Mito
import React from 'react';
import { AgGridReact } from 'ag-grid-react';
import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-alpine.css';
import '../../css/mitosheet.css';
import MitoCellEditor from './MitoCellEditor';
import { TAKE_SUGGESTION_KEYS, ARROW_UP_KEYS, ARROW_DOWN_KEYS } from './MitoCellEditor';
// And functions for building components
import { buildGridData, buildGridColumns } from '../utils/gridData';
import { calculateAndSetHeaderHeight, headerHeightParams } from '../utils/gridStyling';
const MitoSheet = (props) => {
    var _a;
    function onGridReady(params) {
        if (window.gridApiMap === undefined) {
            window.gridApiMap = new Map();
        }
        window.gridApiMap.set(props.model_id, params.api);
        calculateAndSetHeaderHeight(props.model_id);
    }
    /*
        Ag-grid does some sort of caching that means if we rename a column
        it will treat this as a new column that got added to the _end_.

        This function runs whenever the columns change.
        
        It makes sure that the columns are in the correct order. If they aren't,
        it then places them in the correct order.
    */
    function onGridColumnsChanged(params) {
        if (props.sheetJSON === undefined) {
            return;
        }
        const columnState = params.columnApi.getColumnState();
        const correctColumnIDsOrder = props.sheetJSON.columnIDs;
        for (let i = 0; i < correctColumnIDsOrder.length; i++) {
            const columnID = correctColumnIDsOrder[i].toString();
            // If, ignoring the index column, this column is not in the right place, then
            // we have some reordering to do!
            const correctIndex = i + 1;
            const currentDisplayIndex = columnState.findIndex((columnState) => {
                return columnState.colId === columnID;
            });
            if (correctIndex !== currentDisplayIndex) {
                params.columnApi.moveColumn(columnID, correctIndex);
            }
        }
    }
    /*
        Listener that is run anytime a column resizes. If this resize is caused
        by a user dragging the column, then we save the final width the user
        has dragged it to.
    */
    const onColumnResized = (e) => {
        // We save the resize when it occurs because the user finishes dragging a column!
        if (e.finished && e.source == 'uiColumnDragged') {
            const columns = e.columns;
            if (columns) {
                const resizedColumn = columns[0].getColId();
                const newWidth = columns[0].getActualWidth();
                props.setColumnWidth(props.selectedSheetIndex, resizedColumn, newWidth);
            }
        }
    };
    const columns = buildGridColumns(props.sheetJSON, (_a = props.sheetJSON) === null || _a === void 0 ? void 0 : _a.columnIDs, props.columnIDsMap, props.columnSpreadsheetCodeJSON, props.columnFiltersJSON, props.columnMitoTypes, props.formulaBarValue, props.editingColumnID, props.selectedColumnID, props.cursorIndex, props.setEditingMode, props.setEditingFormula, props.setCursorIndex, props.setCurrOpenTaskpane, props.columnWidths, props.setSelectedColumn, props.setSelectedColumnControlPanelTab);
    const rowData = buildGridData(props.sheetJSON);
    const frameworkComponents = {
        simpleEditor: MitoCellEditor,
    };
    return (React.createElement("div", { className: "ag-theme-alpine mitosheet-ag-grid" },
        React.createElement(AgGridReact, { onGridReady: onGridReady, onGridColumnsChanged: onGridColumnsChanged, onCellFocused: (e) => props.cellFocused(e), onColumnMoved: (e) => props.columnMoved(e), onDragStopped: () => props.columnDragStopped(), overlayNoRowsTemplate: '<span style="font-size: 20px; color: #343434;">  Import data to start using Mito. <br /> Use the import button in the Mitosheet toolbar above.</span>', rowData: rowData, rowHeight: 25, rowSelection: 'multiple', headerHeight: headerHeightParams.defaultHeaderHeight, frameworkComponents: frameworkComponents, suppressKeyboardEvent: (params) => {
                /*
                    While we're editing a cell, we suppress events that we use
                    to do things within the editor.

                    NOTE: this function should suppress the events matched in onKeyDown
                    in MitoCellEditor!
                */
                if (!params.editing) {
                    return false;
                }
                return TAKE_SUGGESTION_KEYS.includes(params.event.key) ||
                    ARROW_UP_KEYS.includes(params.event.key) ||
                    ARROW_DOWN_KEYS.includes(params.event.key);
            }, suppressDragLeaveHidesColumns: true, suppressColumnMoveAnimation: true, onColumnResized: onColumnResized }, columns)));
};
export default MitoSheet;
//# sourceMappingURL=MitoSheet.js.map