// Copyright (c) Mito
import React, { forwardRef } from 'react';
import '../../../css/elements/Input.css';
import { classNames } from '../../utils/classNames';
/**
 * The input component is a text field that allows the user
 * to input a single item. It is meant for a single line of text,
 * like an email!
 *
 * Note that this component uses the forwardRef pattern, as is
 * explained here: https://reactjs.org/docs/forwarding-refs.html
 *
 * At a high-level, this allows us to have parent components access
 * refs for the input (which is necessary as we sometimes need the ref
 * to focus on this input).
 */
const Input = forwardRef(function Input(props, 
/**
    * @param [ref] - An optional ref that is set by the input field so that the creating component can access it
*/
ref) {
    // Create better default values to handle optional params 
    const width = props.width || 'block';
    const widthClass = `element-width-${width}`;
    return (React.createElement("input", { className: classNames('input', 'text-body-2', widthClass), value: props.value, placeholder: props.placeholder, onChange: props.onChange, autoFocus: props.autoFocus, required: props.required, disabled: props.disabled, ref: ref }));
});
export default Input;
//# sourceMappingURL=Input.js.map