// Copyright (c) Mito
import React from 'react';
import Input from '../../elements/Input';
import XIcon from '../../icons/XIcon';
import Col from '../../spacing/Col';
import Row from '../../spacing/Row';
import { TaskpaneType } from '../taskpanes';
/*
    A modal that allows a user to sort a column
*/
class ColumnNameCard extends React.Component {
    constructor(props) {
        super(props);
        this.updateUnsubmittedNewColumnHeader = (e) => {
            const newColumnHeader = e.target.value;
            // Update the saved unsubmittedColumnHeader
            this.setState({
                unsubmittedNewColumnHeader: newColumnHeader
            });
        };
        this.changeColumnHeader = async () => {
            // If the column header did not change, just stop renaming without updating the name to avoid error
            if (this.state.columnHeader == this.state.unsubmittedNewColumnHeader) {
                this.setState({
                    isRename: false
                });
                return;
            }
            const stepID = await this.props.mitoAPI.sendRenameColumn(this.props.selectedSheetIndex, this.props.columnID, this.state.unsubmittedNewColumnHeader, this.state.stepID);
            this.setState(prevState => {
                return {
                    stepID: stepID,
                    isRename: false,
                    columnHeader: prevState.unsubmittedNewColumnHeader,
                    unsubmittedNewColumnHeader: prevState.unsubmittedNewColumnHeader
                };
            });
            this.props.setCurrOpenTaskpane({ type: TaskpaneType.CONTROL_PANEL, openEditingColumnHeader: false });
        };
        // We turn any column header into a string, for simplicity.
        // TODO: in the future, we might want to handle allow users to write numbers?
        const columnHeader = props.columnIDsMapArray[props.selectedSheetIndex][props.columnID].toString();
        this.state = {
            columnHeader: columnHeader,
            unsubmittedNewColumnHeader: columnHeader,
            isRename: props.openEditingColumnHeader,
            stepID: ''
        };
        this.setColumnHeader = this.setColumnHeader.bind(this);
        this.updateUnsubmittedNewColumnHeader = this.updateUnsubmittedNewColumnHeader.bind(this);
        this.setIsRename = this.setIsRename.bind(this);
    }
    setColumnHeader(columnHeader) {
        this.setState({
            columnHeader: columnHeader
        });
    }
    setIsRename(isRename) {
        this.setState(prevState => {
            return {
                isRename: isRename,
                unsubmittedNewColumnHeader: prevState.columnHeader
            };
        });
        /*
            When the user changes renaming mode, it should be updated
            in the Mito state so that when they change columns, that
            state change if reflected.
        */
        this.props.setColumnRenameOpenOrClosed(isRename);
    }
    render() {
        return (React.createElement(React.Fragment, null,
            React.createElement(Row, { justify: 'space-between', align: 'center' },
                React.createElement(Col, { span: 20 },
                    this.state.isRename &&
                        React.createElement("form", { className: 'element-width-block', onSubmit: (e) => { e.preventDefault(); void this.changeColumnHeader(); }, onBlur: (e) => {
                                e.preventDefault();
                                void this.changeColumnHeader();
                                this.setIsRename(false);
                            } },
                            React.createElement(Input, { value: this.state.unsubmittedNewColumnHeader, onChange: (e) => { this.updateUnsubmittedNewColumnHeader(e); }, autoFocus: true })),
                    !this.state.isRename &&
                        React.createElement("p", { className: 'text-header-2 text-overflow-hide text-underline-on-hover', onClick: () => {
                                this.setIsRename(!this.state.isRename);
                            } }, this.state.columnHeader)),
                React.createElement(Col, { offset: 1 },
                    React.createElement(XIcon, { onClick: () => this.props.setCurrOpenTaskpane({ type: TaskpaneType.NONE }) })))));
    }
}
export default ColumnNameCard;
//# sourceMappingURL=ColumnNameCard.js.map