// Copyright (c) Mito
import React, { useEffect, useState } from 'react';
import Select from '../../../elements/Select';
import Col from '../../../spacing/Col';
import Row from '../../../spacing/Row';
// The dtypes accepted by the change column dtype change
var ColumnDtypes;
(function (ColumnDtypes) {
    ColumnDtypes["BOOL"] = "bool";
    ColumnDtypes["INT"] = "int";
    ColumnDtypes["FLOAT"] = "float";
    ColumnDtypes["STRING"] = "str";
    ColumnDtypes["DATETIME"] = "datetime";
    ColumnDtypes["TIMEDELTA"] = "timedelta";
})(ColumnDtypes || (ColumnDtypes = {}));
/*
    Given the raw dtype of the column (e.g. 'int64'), gets
    the dtype that will choosen in the value dropdown for
    that column.
*/
function getDtypeValue(dtype) {
    if (dtype === undefined) {
        return ColumnDtypes.STRING;
    }
    if (dtype.includes('bool')) {
        return ColumnDtypes.BOOL;
    }
    else if (dtype.includes('int')) {
        return ColumnDtypes.INT;
    }
    else if (dtype.includes('float')) {
        return ColumnDtypes.FLOAT;
    }
    else if (dtype.includes('str') || dtype.includes('object')) {
        return ColumnDtypes.STRING;
    }
    else if (dtype.includes('datetime')) {
        return ColumnDtypes.DATETIME;
    }
    else if (dtype.includes('timedelta')) {
        return ColumnDtypes.TIMEDELTA;
    }
    return ColumnDtypes.STRING;
}
/*
    A card that allows a user to change
    the dtype of a column, if it is not
    a formula column.

    Otherwise, if it is a formula column,
    just displays the dtype.
*/
function DtypeCard(props) {
    const [stepID, setStepID] = useState(undefined);
    const [columnDtype, setColumnDtype] = useState(undefined);
    async function loadColumnDtype() {
        const loadedColumnDtype = await props.mitoAPI.getColumnDtype(props.selectedSheetIndex, props.columnID);
        setColumnDtype(loadedColumnDtype);
    }
    async function changeColumnDtype(newDtype) {
        const newStepID = await props.mitoAPI.changeColumnDtype(props.selectedSheetIndex, props.columnID, newDtype, stepID);
        setStepID(newStepID);
        await loadColumnDtype();
    }
    useEffect(() => {
        void loadColumnDtype();
        // NOTE: we reload the type when the column formula changes
        // because there is a chance that changing the formula
        // changes the column type
    }, [props.columnFormula, props.lastStepIndex]);
    return (React.createElement(React.Fragment, null,
        React.createElement(Row, { justify: 'space-between', align: 'center' },
            React.createElement(Col, { span: 4 },
                React.createElement("p", { className: 'text-header-3' }, "Dtype")),
            React.createElement(Col, { offset: 2, flex: '1' },
                columnDtype !== undefined && props.columnFormula === '' &&
                    React.createElement(Select, { value: getDtypeValue(columnDtype), options: [
                            ColumnDtypes.BOOL,
                            ColumnDtypes.INT,
                            ColumnDtypes.FLOAT,
                            ColumnDtypes.STRING,
                            ColumnDtypes.DATETIME,
                            ColumnDtypes.TIMEDELTA
                        ], onChange: (newDtype) => {
                            void changeColumnDtype(newDtype);
                        } }),
                columnDtype !== undefined && props.columnFormula !== '' &&
                    React.createElement("p", { className: 'text-header-3 text-align-right' }, getDtypeValue(columnDtype)))),
        props.columnFormula !== '' &&
            React.createElement(Row, null,
                React.createElement(Col, null,
                    React.createElement("p", { className: 'text-subtext-1' }, "Edit the formula to DATETIME, VALUE, or TEXT to change the type.")))));
}
export default DtypeCard;
//# sourceMappingURL=DtypeCard.js.map