// Copyright (c) Mito
import React from 'react';
import '../../../../css/taskpanes/Documentation/DocumentationTaskpane.css';
import { functionDocumentationObjects } from '../../../data/function_documentation';
import { basicExamples } from './BasicExampleContainer';
import { TaskpaneType } from '../taskpanes';
import { FeedbackOpenLocation } from '../Feedback/FeedbackTaskpane';
import TextButton from '../../elements/TextButton';
import DocumentationListIcon from '../../icons/DocumentationListIcon';
/*
    This documentation taskpane content displays a list of all
    documentation entries users can select.

    These are either:
    1. Basic examples (e.g. how to write a formula)
    2. Function documentations
*/
const DocumentationListContent = (props) => {
    const basicExamplesList = basicExamples.map((basicExample) => {
        return (React.createElement("li", { className: 'documentation-taskpane-content-function-list-element', key: basicExample.basicExampleName, onClick: () => {
                // Log the selection of the basic documentation
                // NOTE: we don't need to wait on this promise, as we don't care when
                // it finishes
                void props.mitoAPI.sendLogMessage('documentation_log_event', {
                    kind: 'basic',
                    function: basicExample.basicExampleName
                });
                // Set the documentation element
                props.setSelectedDocumentation({
                    kind: 'basic',
                    basicExampleName: basicExample.basicExampleName
                });
            } },
            React.createElement("div", { className: 'documentation-taskpane-content-function-list-icon' },
                React.createElement(DocumentationListIcon, null)),
            basicExample.basicExampleName));
    });
    const functionNameList = functionDocumentationObjects.map((funcDocObject) => {
        return (React.createElement("li", { className: 'documentation-taskpane-content-function-list-element', key: funcDocObject.function, onClick: () => {
                // Log they selected this function
                // NOTE: we don't need to wait on this promise, as we don't care
                // when it finishes
                void props.mitoAPI.sendLogMessage('documentation_log_event', {
                    kind: 'function',
                    function: funcDocObject.function
                });
                // Set the documentation
                props.setSelectedDocumentation({
                    kind: 'function',
                    function: funcDocObject.function
                });
            } },
            React.createElement("div", { className: 'documentation-taskpane-content-function-list-icon' },
                React.createElement(DocumentationListIcon, null)),
            funcDocObject.function));
    });
    return (React.createElement(React.Fragment, null,
        React.createElement("div", null,
            React.createElement(TextButton, { variant: 'dark', width: 'block', onClick: () => { props.setCurrOpenTaskpane({ type: TaskpaneType.FEEDBACK, openLocation: FeedbackOpenLocation.FROM_DOCUMENTATION }); } }, "Provide Feedback")),
        React.createElement("div", { className: 'documentation-taskpane-content-function-list-title mt-10px' }, "Basic Examples"),
        React.createElement("ul", { className: 'documentation-taskpane-content-function-list' }, basicExamplesList),
        React.createElement("div", { className: 'documentation-taskpane-content-function-list-section-title' }, "Functions"),
        React.createElement("ul", { className: 'documentation-taskpane-content-function-list' }, functionNameList)));
};
export default DocumentationListContent;
//# sourceMappingURL=DocumentationListContent.js.map