// Copyright (c) Mito
import { getKeyFromValue } from '../../../utils/objects';
/*
    Returns all column headers shared between the sheets at the given indexes.
*/
const getSharedColumnHeaders = (columnIDsMapArray, sheetOneIndex, sheetTwoIndex) => {
    // If invalid indexes are passed, return early to avoid errors
    if ((sheetOneIndex < 0 || columnIDsMapArray.length <= sheetOneIndex) || (sheetTwoIndex < 0 || columnIDsMapArray.length <= sheetTwoIndex)) {
        return [];
    }
    const sheetOneColumnNames = Object.values(columnIDsMapArray[sheetOneIndex]);
    const sheetTwoColumnNames = Object.values(columnIDsMapArray[sheetTwoIndex]);
    const sharedKeys = sheetOneColumnNames.filter(columnName => sheetTwoColumnNames.includes(columnName)).map(columnHeader => columnHeader.toString());
    return sharedKeys;
};
/*
    Given two sheet indexes, reccomends a key to merge on. If there are x shared keys, then:
    - If x.length >= 1, it returns the first shared key as reccomended.
    - Else, it reccomends the first key
*/
export const getSuggestedKeysColumnID = (columnIDsMapArray, sheetOneIndex, sheetTwoIndex) => {
    const sharedKeys = getSharedColumnHeaders(columnIDsMapArray, sheetOneIndex, sheetTwoIndex);
    const sheetOneMergeKeyHeader = sharedKeys.length >= 1 ? sharedKeys[0] : Object.values(columnIDsMapArray[sheetOneIndex])[0];
    const sheetTwoMergeKeyHeader = sharedKeys.length >= 1 ? sharedKeys[0] : Object.values(columnIDsMapArray[sheetTwoIndex])[0];
    return {
        mergeKeyColumnIDOne: getKeyFromValue(columnIDsMapArray[sheetOneIndex], sheetOneMergeKeyHeader),
        mergeKeyColumnIDTwo: getKeyFromValue(columnIDsMapArray[sheetTwoIndex], sheetTwoMergeKeyHeader)
    };
};
//# sourceMappingURL=mergeUtils.js.map