/*
    Helper function that determines how much we should shift the dropdown down.

    To do so, we find the bounding box of the Mito container as well as the saved analysis
    card itself. The bounding box can be understood as the left, right, top and bottom pixel indexes
    of the div element itself.

    We return the difference between the top edge of the mito container and the saved analysis card.
    However, we cap this at 1, so that the dropdown must be displayed within the Mito container.

    We use this approach of setting the dropdown location because the dropdown's z-index is limited by
    the z-index of the nearest parent that has a position set. To resolve this we could either move
    the dropdown into the replay analysis taskpane itself, but doing so requires a huge refactor
    that moves most of the savedAnalysisCardlogic into the ReplayAnalysisTaskpane itself. It's quite ugly.

    Alternativly, we opted to remove the "position: relative" css from the SavedAnalysisCard
    which allows the z-index to take effect on the ReplayAnalysisTaskpane instead. However, doing so
    makes the dropdown displayed relative to the replayAnalysisTaskpane isntead of the scrollable list
    of SavedAnalysisCards. So we must calculate the correct offset to apply to the dropdown.

    See a full explanation here: https://www.freecodecamp.org/news/4-reasons-your-z-index-isnt-working-and-how-to-fix-it-coder-coder-6bc05f103e6c/
*/
export function getVerticalShift(mitoContainerRef, elementRef) {
    var _a, _b, _c;
    const containerRectTop = (_b = (_a = elementRef.current) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect()) === null || _b === void 0 ? void 0 : _b.top;
    const savedAnalysisCardRectTop = (_c = mitoContainerRef === null || mitoContainerRef === void 0 ? void 0 : mitoContainerRef.getBoundingClientRect()) === null || _c === void 0 ? void 0 : _c.top;
    if (containerRectTop !== undefined && savedAnalysisCardRectTop !== undefined) {
        return Math.max(containerRectTop - savedAnalysisCardRectTop, 1);
    }
    return 0;
}
//# sourceMappingURL=getVerticalShift.js.map